#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <ctime>
#include <string>

#include "Meteor.h"
#include "Star.h"
#include "StationarySprite.h"
#include "CosmicDustParticle.h"

void displayWelcome(sf::RenderWindow& window);

void delay()
{
    for (unsigned long long i = 0; i < 100000; ++i);
}

sf::Vector2f BackgroundSize;
sf::Texture StarTexture;

// const std::string ResourcePath = "resources/";
const std::string ResourcePath = "E:/SFML/wandering_star/resources/";

int main()
{
    srand(static_cast<unsigned>(time(0)));
    unsigned long loopCounter = 0;
    const unsigned CosmicDustCount = 100;

    sf::RenderWindow window(sf::VideoMode(1024, 768), "Wandering Star", sf::Style::Close | sf::Style::Resize);

    // Background
    sf::RectangleShape background(sf::Vector2f(1024.0f, 768.0f));
    sf::Texture backgroundTexture;
    BackgroundSize = background.getSize();
    backgroundTexture.loadFromFile(ResourcePath + "dark_sky.png");
    background.setTexture(&backgroundTexture);

    displayWelcome(window);

    // Music
    sf::Music music;
    music.openFromFile(ResourcePath + "testmusic.ogg");
    music.setVolume(40);
    music.play();
    sf::Music chirp;
    chirp.openFromFile(ResourcePath + "credulous.ogg");
    chirp.setVolume(30);
    chirp.setLoop(true);

    sf::SoundBuffer buffer;
    buffer.loadFromFile(ResourcePath + "czshhh.wav");
    sf::Sound sound;
    sound.setBuffer(buffer);

    ////////////////////////  Sprites  //////////////////////////////

    StarTexture.loadFromFile(ResourcePath + "star.png");
    Star star;
    Star::Direction starMoveDirection;

    sf::Texture moonTexture;
    moonTexture.loadFromFile(ResourcePath + "moon.png");
    StationarySprite moon(moonTexture, 0.8f);

    sf::Texture cometTexture;
    cometTexture.loadFromFile(ResourcePath + "comet.png");
    StationarySprite comet(cometTexture, 0.1f);

    CosmicDustParticle cosmicDust[CosmicDustCount];

    Meteor meteor;

    while (window.isOpen())
    {
        delay();
        sf::Event evnt;
        while (window.pollEvent(evnt))
        {
            switch (evnt.type)
            {
            case sf::Event::Closed:
                window.close();
                break;
            case sf::Event::MouseButtonPressed:  // move star with mouse
                star.setPosition(sf::Vector2f(static_cast<float>(evnt.mouseButton.x), static_cast<float>(evnt.mouseButton.y)));
                break;
            case sf::Event::KeyPressed:
                if (sf::Keyboard::isKeyPressed(sf::Keyboard::M))  // meteor
                    loopCounter = 95;
                if (sf::Keyboard::isKeyPressed(sf::Keyboard::C))  // move cosmic dust
                    loopCounter = 999;
                if (sf::Keyboard::isKeyPressed(sf::Keyboard::Right))  // move moon to the right
                    moon.move(StationarySprite::Right);
                if (sf::Keyboard::isKeyPressed(sf::Keyboard::Left))  // move moon to the left
                    moon.move(StationarySprite::Left);
                if (sf::Keyboard::isKeyPressed(sf::Keyboard::Up))  // move moon to the right
                    comet.move(StationarySprite::Up);
                if (sf::Keyboard::isKeyPressed(sf::Keyboard::Down))  // move moon to the left
                    comet.move(StationarySprite::Down);
                break;

            default:
                ;
            }
        }

        ///////////////////  star movement  ////////////////////
        if (loopCounter%10 == 0)
        {
            starMoveDirection = star.move();
            // look for collisions
            if (star.collideWithSprite(moon.getSprite()))
            {
                star.move(star.getOppositeDirection(starMoveDirection));
                std::cout << "Collision with moon" << std::endl;
            }
            if (star.collideWithSprite(comet.getSprite()))
            {
                star.move(star.getOppositeDirection(starMoveDirection));
                std::cout << "Collision with comet" << std::endl;
            }
        }

        // star collisions with cosmic dust particles
        for (auto i = 0u; i < CosmicDustCount; i++)
        {
            if (star.collideWithSprite(cosmicDust[i].getParticle()))
            {
                std::cout << "Cosmic dust particle eaten" << std::endl;
                cosmicDust[i].setAttributes();
                break;
            }
        }

        ///////// Move a random cosmic dust particle  ///////////
        if (loopCounter % 1000 == 0)
        {
            cosmicDust[rand()%CosmicDustCount].setAttributes();
        }

        //////////////  Redraw the window  //////////////////////
        window.clear();
        window.draw(background);
        window.draw(star.getStar());
        window.draw(moon.getSprite());
        window.draw(comet.getSprite());

        //////////////////////// Meteor  ////////////////////////
        if (loopCounter % 5000 == 100)
        {
            sound.play();
            meteor.assignNewPosition();
        }
        if (loopCounter % 5000 > 100 && loopCounter % 5000 < 500)
        {
            meteor.move();
            window.draw(meteor.get_meteor());
        }

        ///////////////////// Cosmic Dust ///////////////////////
        for (auto i=0u; i < CosmicDustCount; i++)
        {
            window.draw(cosmicDust[i].getParticle());
        }

        window.display();

        loopCounter++;
    }
}

void displayWelcome(sf::RenderWindow& window)
{
    std::string textFromFile, buffer;
    std::ifstream fin(ResourcePath + "welcome.txt");
    if (!fin)
    {
        std::cout << "Cannot open welcome file." << std::endl;
        return;
    }
    while (getline(fin,buffer))
        textFromFile += buffer += '\n';

    fin.close();

    // Text
    sf::Font font;
    if (!font.loadFromFile(ResourcePath + "OpenSans-Light.ttf"))
    {
        std::cout << "Can't find font" << std::endl;
    }
    sf::Text text;
    text.setFont(font);
    text.setString(textFromFile.c_str());
    // set the character size
    text.setCharacterSize(24); // in pixels, not points!
    // set the color
    text.setFillColor(sf::Color::Red);
    // set the text style
    text.setStyle(sf::Text::Bold);
    text.setPosition(100.0f,50.0f);

    sf::Event evnt;
    while (window.isOpen())
    {
        while (window.pollEvent(evnt))
        {
            switch (evnt.type)
            {
            case sf::Event::Closed:
                window.close();
                break;
            case sf::Event::KeyPressed:
                if (sf::Keyboard::isKeyPressed(sf::Keyboard::Space))
                {
                    window.clear();
                    return;
                }
            default:
                ;
            }
        }
        window.clear();
        window.draw(text);
        window.display();
    }
}

